@@ -28,10 +28,10 @@ import re
 
 from parce import Language, lexicon, skip, default_action, default_target
 from parce.action import (
-    Bracket, Character, Comment, Delimiter, Direction, Keyword, Name, Number,
-    Operator, Separator, String, Text)
+    Bracket, Character, Comment, Delimiter, Direction, Keyword, Literal, Name,
+    Number, Operator, Separator, String, Text)
 from parce.rule import (
-    MATCH, TEXT, arg, bygroup, call, dselect, findmember, ifeq, ifgroup,
+    MATCH, TEXT, arg, bygroup, call, dselect, findmember, ifarg, ifeq, ifgroup,
     ifmember, select, words)
 
 from . import lilypond_words
@@ -45,19 +45,14 @@ RE_LILYPOND_ID_RIGHT_BOUND = r"(?![_-]?[^\W\d_])"
 RE_LILYPOND_ID = r"[^\W\d_]+(?:[_-][^\W\d_]+)*"
 RE_LILYPOND_SYMBOL = RE_LILYPOND_ID + RE_LILYPOND_ID_RIGHT_BOUND
 RE_LILYPOND_COMMAND = r"\\(" + RE_LILYPOND_ID + ")" + RE_LILYPOND_ID_RIGHT_BOUND
-RE_LILYPOND_MARKUP_TEXT = r'[^{}"\\\s$#]+'
-RE_LILYPOND_LYRIC_TEXT = r'[^{}"\\\s$#\d]+'
-RE_LILYPOND_DYNAMIC = \
-    r"\\(?:[<!>]|(?:{})(?![A-Za-z]))".format(words(lilypond_words.dynamics))
+RE_LILYPOND_MARKUP_TEXT = r'[^%{}"\\\s$#][^{}"\\\s$#]*'
+RE_LILYPOND_LYRIC_TEXT = r'[^%={}"\\\s$#\d][^{}"\\\s$#\d]*'
 
 RE_LILYPOND_REST = r"[rRs](?![^\W\d])"
 
 # a string that could be a valid pitch name (or drum name)
 RE_LILYPOND_PITCHWORD = r"(?<![^\W\d])[a-zé]+(?:[_-][a-zé]+)*(?![^\W\d_])"
 
-# a pitch name followed by an optional octave (two capturing groups)
-RE_LILYPOND_PITCH_OCT = "(" + RE_LILYPOND_PITCHWORD + r")\s*('+|,+)?"
-
 # all durations
 RE_LILYPOND_DURATION = words(lilypond_words.durations, suffix = r'(?!\d)')
 
@@ -95,10 +90,16 @@ class LilyPond(Language):
     def root(cls):
         """Toplevel LilyPond document."""
         yield from cls.blocks()
-        yield RE_LILYPOND_SYMBOL, Name.Variable.Definition, cls.identifier
+        yield RE_LILYPOND_SYMBOL, findmember(TEXT, (
+                (lilypond_words.all_pitch_names, (Pitch, cls.pitch)),
+                (lilypond_words.contexts, (Context, cls.identifier)),
+                (lilypond_words.grobs, (Grob, cls.identifier))),
+              (Name.Variable.Definition, cls.identifier))
         yield from cls.find_string(cls.identifier)
-        yield "=", Operator.Assignment
+        yield '=', Operator.Assignment, cls.start_list
         yield from cls.music()
+        yield from cls.common()
+        yield from cls.commands(list_target=cls.start_list)
 
     @classmethod
     def blocks(cls):
@@ -112,46 +113,41 @@ class LilyPond(Language):
     @lexicon(consume=True)
     def book(cls):
         """Book or bookpart."""
-        yield r'\}', Bracket.End, -1
-        yield from cls.common()
-        yield from cls.blocks()
-        yield from cls.music()
+        yield from cls.score
 
     @lexicon(consume=True)
     def score(cls):
-        """A score block, can also occur inside markup."""
+        """A score block."""
         yield r'\}', Bracket.End, -1
         yield from cls.blocks()
         yield from cls.music()
+        yield from cls.common()
+        yield from cls.commands()
 
     @lexicon(consume=True)
     def header(cls):
         """A header block."""
         yield r'\}', Bracket.End, -1
-        yield RE_LILYPOND_SYMBOL, Name.Variable, cls.identifier
-        yield "=", Operator.Assignment
+        yield RE_LILYPOND_SYMBOL, Name.Attribute, cls.identifier
+        yield RE_FRACTION, Number.Fraction
+        yield r'\d+\.\d+', Number.Float
         yield from cls.common()
+        yield from cls.commands()
 
     @lexicon(consume=True)
     def paper(cls):
         """A paper block."""
-        yield r'\}', Bracket.End, -1
-        yield RE_LILYPOND_SYMBOL, Name.Variable, cls.identifier
-        yield "=", Operator.Assignment
-        yield r'\d+', Number, cls.unit
-        yield RE_FRACTION, Number
-        yield from cls.common()
-        yield from cls.commands()
+        yield from cls.header
 
     @lexicon(consume=True)
     def layout(cls):
         """A layout block."""
         yield r'\}', Bracket.End, -1
-        yield RE_LILYPOND_SYMBOL, Name.Variable, cls.identifier
-        yield "=", Operator.Assignment
-        yield r'\d+', Number, cls.unit
+        yield RE_LILYPOND_SYMBOL, Name.Attribute, cls.identifier
         yield r"(\\context)\s*(\{)", bygroup(Keyword, Bracket.Start), cls.layout_context
         yield from cls.music()
+        yield from cls.common()
+        yield from cls.commands()
 
     @lexicon(consume=True)
     def midi(cls):
@@ -160,51 +156,60 @@ class LilyPond(Language):
 
     @lexicon(consume=True)
     def layout_context(cls):
-        r"""Contents of \layout or \midi { \context { } } or \with. { }."""
-        yield r'\}', Bracket.End, -1
-        yield words(lilypond_words.contexts, prefix=r"\\", suffix=r"\b"), Context
-        yield words(lilypond_words.grobs), Grob
-        yield words(lilypond_words.keywords, prefix=r"\\", suffix=r"(?![^\W\d])"), Keyword
-        yield RE_LILYPOND_SYMBOL, Name.Variable, cls.identifier
-        yield "=", Operator.Assignment
-        yield r'\d+', Number, cls.unit
-        yield from cls.common()
-        yield from cls.commands()
+        r"""Contents of ``\layout`` or ``\midi { \context { } }`` or ``\with. { }``."""
+        yield RE_LILYPOND_SYMBOL, findmember(TEXT, (
+                (lilypond_words.contexts, Context),
+                (lilypond_words.grobs, Grob)), (Name.Variable, cls.identifier))
+        yield from cls.header
 
     # ------------------ commands that can occur in all input modes --------
     @classmethod
-    def commands(cls):
-        """Yield commands that can occur in all input modes."""
-        yield RE_LILYPOND_DYNAMIC, Dynamic
-        yield r"(\\repeat)\b(?:\s+([a-z]+)\s*(\d+)?)?", bygroup(Keyword, Name.Symbol, Number)
-        yield r"\\(?:un)?set(?![^\W\d])", Keyword, cls.set_unset
-        yield r"\\(?:override|revert)(?![^\W\d])", Keyword, cls.override
-        yield r"\\(?:new|change|context)(?![^\W\d])", Keyword, cls.context
+    def commands(cls, /, list_target=0):
+        """Yield commands that can occur in all input modes.
+
+        If a ``list_target`` is given, that lexicon is pushed after a Keyword,
+        to be able to parse symbols, strings, numbers or scheme. This makes
+        sense in e.g. lyrics mode.
+
+        """
         yield r"(\\with)\s*(\{)", bygroup(Keyword, Bracket.Start), cls.layout_context
-        yield r"(\\key)(?![^\W\d])(?:\s+(" + RE_LILYPOND_PITCHWORD +"))?", \
-            bygroup(Name.Builtin, cls.ifpitch())
-        yield r"(\\relative)(?![^\W\d])(?:\s+" + RE_LILYPOND_PITCH_OCT + ")?", \
-            bygroup(Name.Builtin, cls.ifpitch(), Octave)
-        yield (r"(\\transpose)(?![^\W\d])(?:\s+" + RE_LILYPOND_PITCH_OCT + ")?"
-               r"(?:\s*" + RE_LILYPOND_PITCH_OCT + ")?"), \
-               bygroup(Name.Builtin, cls.ifpitch(), Octave, cls.ifpitch(), Octave)
-        yield r"\\tempo(?![^\W\d])", Name.Builtin, cls.tempo
-        yield r"(\\chord(?:s|mode))\b\s*(\{)?", bygroup(Keyword, Bracket.Start), \
-            ifgroup(2, cls.chordmode)
-        yield from cls.notemode_rule()
-        yield from cls.lyricmode_rules()
-        yield from cls.drummode_rule()
-        yield RE_LILYPOND_COMMAND, findmember(MATCH[1], (
-            (lilypond_words.keywords, Keyword),
-            (lilypond_words.music_commands, Name.Builtin),
-            (lilypond_words.all_articulations, Articulation),
-            ), Name.Command)
+        yield r'(' + RE_LILYPOND_COMMAND + r')(?=\s*(\.))?', ifgroup(3,
+            # part of a \bla.bla or \bla.1 construct, always a user command
+            (Name.Variable, cls.identifier_ref),
+            # no "." or "," , can be a builtin
+            dselect(MATCH[2], {
+                # input modes
+                "lyricsto": (Keyword.Lyric, cls.lyricsto, cls.start_list),
+                "addlyrics": (Keyword.Lyric, cls.lyricmode),
+                "lyrics": (Keyword.Lyric, cls.lyricmode),
+                "lyricmode": (Keyword.Lyric, cls.lyricmode),
+                "chords": (Keyword, cls.chordmode),
+                "chordmode": (Keyword, cls.chordmode),
+                "drums": (Keyword, cls.drummode),
+                "drummode": (Keyword, cls.drummode),
+                "figures": (Keyword, cls.figuremode),
+                "figuremode": (Keyword, cls.figuremode),
+                "notemode": (Keyword, cls.notemode),    # \notes doesn't exist anymore
+                # commands that expect some symbols in all input modes
+                "repeat": (Keyword, cls.repeat),
+            },
+            (findmember(MATCH[2], (
+                (lilypond_words.keywords, (Keyword, list_target)),
+                (lilypond_words.all_articulations, Articulation),
+                (lilypond_words.dynamics, Dynamic),
+                (lilypond_words.music_commands, Name.Builtin),
+                (lilypond_words.contexts, Name.Builtin.Context),
+                (lilypond_words.modes, Name.Type),
+            ), (Name.Variable, cls.identifier_ref)))))
+        # seldom used, but nevertheless allowed in LilyPond: \"blabla"
+        yield r'(\\)(?=")', Name.Variable, cls.identifier_ref
 
     # ------------------ music ----------------------
     @classmethod
     def music(cls):
         """Musical items."""
-        yield from cls.common()
+        yield from cls.find_string(cls.list)
+        yield from cls.find_scheme(cls.list)
         yield r"\{", Bracket.Start, cls.musiclist('}')
         yield r"<<", Bracket.Start, cls.musiclist('>>')
         yield r"<", Delimiter.Chord.Start, cls.chord
@@ -216,76 +221,53 @@ class LilyPond(Language):
         yield r"[()]", Spanner.Slur
         yield r"~", Spanner.Tie
         yield r"\\~", Spanner.PesOrFlexa
+        yield r"\\[<>!]", Dynamic
         yield r"[-_^]", Direction, cls.script
         yield r"(\\=)\s*(?:(\d+)|({}))?".format(RE_LILYPOND_SYMBOL), \
             bygroup(Spanner.Id, Number, cls.ifpitch(Name.Symbol.Invalid, Name.Symbol))
         yield r"q(?![^\W\d])", Pitch
         yield RE_LILYPOND_REST, Rest
-        yield RE_LILYPOND_PITCHWORD, cls.ifpitch((Pitch, cls.pitch))
-        yield words(lilypond_words.contexts), Context
-        yield words(lilypond_words.grobs), Grob
+        yield RE_LILYPOND_SYMBOL, findmember(TEXT, (
+                (lilypond_words.all_pitch_names, (Pitch, cls.pitch)),
+                (lilypond_words.contexts, (Context, cls.list)),
+                (lilypond_words.grobs, (Grob, cls.list))), (Name.Symbol, cls.list))
         yield r'[.,]', Delimiter
         yield r'(:)\s*(8|16|32|64|128|256|512|1024|2048)?(?!\d)', bygroup(Delimiter.Tremolo, Duration.Tremolo)
-        yield RE_FRACTION, Number
+        yield RE_FRACTION, Number.Fraction
+        yield r'\d+\.\d+', Number.Float
+        yield r'(\d+)(?=\s*,)', Number, cls.list
         yield RE_LILYPOND_DURATION, Duration, cls.duration
-        yield r"\d+", Number
-        yield from cls.commands()
+        yield r"\d+", Number, cls.list
 
     @lexicon(consume=True)
     def musiclist(cls):
-        """A { } or << >> construct; derive with the end arg (} or >>)."""
+        """A ``{`` ... ``}`` or ``<<`` ... ``>>`` musical construct.
+
+        Derive with the end arg (``}`` or ``>>``).
+
+        """
         yield arg(), Bracket.End, -1
         yield from cls.music()
+        yield from cls.common()
+        yield from cls.commands()
 
     @lexicon(consume=True)
     def chord(cls):
-        """A < chord > construct."""
+        """A ``<`` chord ``>`` construct."""
         yield r">", Delimiter.Chord.End, -1
         yield from cls.music()
-
-    # ------------------ special commands ---------------
-    @lexicon(consume=True)
-    def tempo(cls):
-        """Find content after a tempo command."""
-        yield SKIP_WHITESPACE
-        yield from cls.common() # markup, scheme, string, comment
-        yield RE_LILYPOND_SYMBOL, Name.Symbol
-        yield RE_LILYPOND_DURATION, Duration, cls.duration
-        yield r"(=)\s*(?:(\d+)(?:\s*(-)\s*(\d+))?)?", bygroup(
-            Operator.Assignment, Number, Operator, Number), -1
-        yield default_target, -1
-
-    @lexicon(consume=True)
-    def context(cls):
-        """\\new, \\change, \\context Context [="name"] stuff."""
-        yield SKIP_WHITESPACE
-        yield words(lilypond_words.contexts), Context
-        yield "=", Operator.Assignment
-        yield RE_LILYPOND_SYMBOL, Name.Symbol
-        yield r"(\\with)\s*(\{)", bygroup(Keyword, Bracket.Start), -1, cls.layout_context
         yield from cls.common()
-        yield default_target, -1
+        yield from cls.commands()
 
-    @lexicon(consume=True)
-    def set_unset(cls):
-        """\\set, \\unset."""
+    # ------------------ repeat -------------------------
+    @lexicon
+    def repeat(cls):
+        """\\repeat mode n."""
         yield SKIP_WHITESPACE
-        yield words(lilypond_words.contexts), Context
-        yield r'[.,]', Delimiter
-        yield r"(=)(?:\s*(" + RE_FRACTION + r"|\d+))?", bygroup(Operator.Assignment, Number)
-        yield RE_LILYPOND_SYMBOL + r"(?=\s*([,.=])?)", Name.Variable.Definition, ifeq(MATCH[1], None, -1)
-        yield default_target, -1
-
-    @lexicon(consume=True)
-    def override(cls):
-        """\\override, \\revert."""
-        yield SKIP_WHITESPACE
-        yield words(lilypond_words.contexts), Context
-        yield words(lilypond_words.grobs), Grob
-        yield r'[.,]', Delimiter
-        yield r"=", Operator.Assignment, -1
-        yield RE_LILYPOND_SYMBOL + r"(?=\s*([,.=])?)", Name.Variable, ifeq(MATCH[1], None, -1)
-        yield from cls.find_scheme()
+        yield words(("volta", "unfold", "percent", "tremolo"), suffix=r'\b'), Name.Type
+        yield from cls.find_string()
+        yield from cls.find_comment()
+        yield r'\d+', Number, -1
         yield default_target, -1
 
     # ------------------ script -------------------------
@@ -334,168 +316,267 @@ class LilyPond(Language):
         yield r"(\*)\s*(\d+(?:/\d+)?)", bygroup(Duration, Duration.Scaling)
         yield default_target, -2
 
+    # --------------------- input modes ------------------
+    @classmethod
+    def inputmode(cls, lexicon):
+        """Yield boilerplate rules for an input mode."""
+        yield SKIP_WHITESPACE
+        yield r"\\s(equential|imultaneous)\b", Keyword
+        yield r"<<", Bracket.Start, -1, lexicon('>>')
+        yield r"\{", Bracket.Start, -1, lexicon('}')
+        yield from cls.find_comment()
+        yield default_target, -1
+
+    @classmethod
+    def inputmode_list(cls, lexicon):
+        """Yield boilerplate rules for the contents of an input mode."""
+        yield arg(), Bracket.End, -1
+        yield r"<<", Bracket.Start, lexicon('>>')
+        yield r"\{", Bracket.Start, lexicon('}')
 
     # --------------------- lyrics -----------------------
-    @classmethod
-    def lyricmode_rules(cls):
-        """Find lyric mode music."""
-        yield r"(\\(?:lyric(?:mode|s)|addlyrics))\b\s*(\\s(?:equential|imultaneous)\b)?\s*(\{|<<)?", \
-            bygroup(Keyword.Lyric, Keyword, Bracket.Start), \
-            dselect(MATCH[3], {'{': cls.lyricmode('}'), '<<': cls.lyricmode('>>')})
-        yield r"\\lyricsto\b", Keyword.Lyric, cls.lyricsto
-
-    @lexicon(consume=True)
+    @lexicon
     def lyricmode(cls):
         """Yield contents in lyric mode."""
-        yield arg(), Bracket.End, -1
-        yield from cls.common()
-        yield r"<<", Bracket.Start, cls.lyricmode('>>')
-        yield r"\{", Bracket.Start, cls.lyricmode('}')
+        yield from cls.inputmode(cls.lyriclist)
+
+    @lexicon
+    def lyricsto(cls):
+        r"""Find the argument of a ``\lyricsto`` command."""
+        yield default_target, -1, cls.lyricmode
+
+    @lexicon(consume=True)
+    def lyriclist(cls):
+        """Lyrics between ``{`` ... ``}``.
+
+        Derive with the desired closing delimiter (``}`` or ``>>``).
+
+        """
+        yield from cls.inputmode_list(cls.lyriclist)
         yield RE_LILYPOND_LYRIC_TEXT, dselect(TEXT, {
                 "--": LyricHyphen,
                 "__": LyricExtender,
                 "_": LyricSkip,
             }, LyricText)
-        yield RE_FRACTION, Number
+        yield RE_FRACTION, Number.Fraction
         yield RE_LILYPOND_DURATION, Duration, cls.duration
-        yield from cls.commands()
-
-    @lexicon(consume=True)
-    def lyricsto(cls):
-        """Find the argument of a \\lyricsto command."""
-        yield from cls.base()
-        yield RE_LILYPOND_SYMBOL, Name.Symbol
-        yield r"\\s(sequential|imultaneous)\b", Keyword
-        yield r"<<", Bracket.Start, -1, cls.lyricmode('>>')
-        yield r"\{", Bracket.Start, -1, cls.lyricmode('}')
-        yield SKIP_WHITESPACE
-        yield default_target, -1
-
-    # ---------------------- notemode ---------------------
-    @classmethod
-    def notemode_rule(cls):
-        """Yield the rule for \\notemode / \\notes."""
-        yield r"(\\note(?:s|mode))\b\s*(\{|<<)?", bygroup(Keyword, Bracket.Start), \
-            dselect(MATCH[2], {
-                "{": (cls.notemode, cls.musiclist('}')),
-                "<<": (cls.notemode, cls.musiclist('>>')),
-            })
-
-    @lexicon
-    def notemode(cls):
-        """\\notemode and \\notes."""
-        yield default_target, -1
+        yield from cls.common()
+        yield from cls.commands(list_target=cls.start_list)
 
     # ---------------------- drummode ---------------------
-    @classmethod
-    def drummode_rule(cls):
-        """Yield the rule for \\drummode / \\drums."""
-        yield r"(\\drum(?:s|mode))\b\s*(\{|<<)?", bygroup(Keyword.Drum, Bracket.Start), \
-            dselect(MATCH[2], {
-                "{": (cls.drummode('}')),
-                "<<": (cls.drummode('>>')),
-            })
-
-    @lexicon(consume=True)
+    @lexicon
     def drummode(cls):
         """\\drummode and \\drums."""
-        yield r"\{", Bracket.Start, cls.drummode('}')
-        yield r"<<", Bracket.Start, cls.drummode('>>')
-        yield arg(), Bracket.End, -1
+        yield from cls.inputmode(cls.drumlist)
+
+    @lexicon(consume=True)
+    def drumlist(cls):
+        """Drum music between ``{`` ... ``}`` or ``<<`` ... ``>>``."""
+        yield from cls.inputmode_list(cls.drumlist)
         yield RE_LILYPOND_REST, Rest
-        yield RE_LILYPOND_PITCHWORD, ifmember(TEXT, lilypond_words.drum_pitch_names_set, Pitch.Drum, Name.Symbol)
+        yield RE_LILYPOND_SYMBOL, findmember(TEXT, (
+                (lilypond_words.drum_pitch_names_set, Pitch.Drum),
+                (lilypond_words.contexts, (Context, cls.list)),
+                (lilypond_words.grobs, (Grob, cls.list))), (Name.Symbol, cls.list))
         yield from cls.music()
+        yield from cls.common()
+        yield from cls.commands()
 
     # ---------------------- chordmode ---------------------
-    @lexicon(consume=True)
+    @lexicon
     def chordmode(cls):
         """\\chordmode and \\chords."""
-        yield r":|/\+?", Separator.Chord, cls.chord_modifier
-        yield r"\}", Bracket.End, -1
-        yield r"\{", Bracket.Start, 1
+        yield from cls.inputmode(cls.chordlist)
+
+    @lexicon(consume=True)
+    def chordlist(cls):
+        """Chordmode music between ``{`` ... ``}`` or ``<<`` ... ``>>``."""
+        yield from cls.inputmode_list(cls.chordlist)
+        yield r"[:^]", Separator.Chord, cls.chord_modifier
+        yield r"(/\+?)\s*(" + RE_LILYPOND_PITCHWORD + ")?", bygroup(Separator.Chord, cls.ifpitch())
         yield from cls.music()
+        yield from cls.common()
+        yield from cls.commands()
 
     @lexicon
     def chord_modifier(cls):
         """Stuff in chord mode after a `:`"""
+        yield SKIP_WHITESPACE
         yield r"((?<![a-z])|^)(?:aug|dim|sus|min|maj|m)(?![a-z])", Name.Symbol
         yield r"(\d+)([-+])?", bygroup(Number, Operator.Alteration)
         yield r"\.", Separator.Dot
-        yield r"/\+?", Separator.Chord
-        yield RE_LILYPOND_PITCHWORD, cls.ifpitch()
+        yield from cls.find_comment()
         yield default_target, -1
 
+    # --------------------- notemode -------------------
+    @lexicon
+    def notemode(cls):
+        """Notemode switches back to music e.g. in lyrics."""
+        yield from cls.inputmode(cls.musiclist)
+
+    # --------------------- figuremode -------------------
+    @lexicon
+    def figuremode(cls):
+        """\\figuremode and \\figures."""
+        yield from cls.inputmode(cls.figurelist)
+
+    @lexicon(consume=True)
+    def figurelist(cls):
+        """figuremode music between ``{`` ... ``}`` or ``<<`` ... ``>>``."""
+        yield from cls.inputmode_list(cls.figurelist)
+        yield r'<', Delimiter.Chord.Start, cls.figure
+        yield RE_LILYPOND_DURATION, Duration, cls.duration
+        yield from cls.music()
+        yield from cls.common()
+        yield from cls.commands()
+
+    @lexicon(consume=True)
+    def figure(cls):
+        """Stuff between ``<`` and ``>`` in figure mode."""
+        yield r'>', Delimiter.Chord.End, -1
+        yield r'\[', Bracket.Start, cls.figurebracket
+        yield from cls._figure_rules()
+
+    @lexicon(consume=True)
+    def figurebracket(cls):
+        """Stuff between ``[`` and ``]`` in a bass figure."""
+        yield r'>', Delimiter.Chord.End, -2
+        yield r'\]', Bracket.End, -1
+        yield from cls._figure_rules()
+
+    @classmethod
+    def _figure_rules(cls):
+        """Rules for figures in [ ] and < >."""
+        yield r'_|\d+', Pitch.Figure
+        yield r'--?|\+\+?|!', Accidental
+        yield r'\\[\\!+]|/', Character.Alteration
+        yield from cls.common()
+
     # -------------------- base stuff --------------------
     @classmethod
     def common(cls):
-        """Find comment, string, scheme and markup."""
+        """Find comment, string, scheme, the ``=`` operator and  markup."""
         yield from cls.base()
-        yield r"\\markup(?:lines|list)?" + RE_LILYPOND_ID_RIGHT_BOUND, Keyword.Markup, cls.markup
+        yield "=", Operator.Assignment, cls.start_list
+        yield from cls.find_markup()
 
     @classmethod
     def base(cls):
         """Find comment, string and scheme."""
-        yield r'"', String, cls.string
+        yield from cls.find_string()
         yield from cls.find_scheme()
         yield from cls.find_comment()
 
     @lexicon(consume=True)
-    def identifier(cls):
-        """bla.bla.bla syntax."""
+    def list(cls):
+        r"""A list of numbers, symbols, strings or scheme expressions.
+
+        Consumes both . and , as separators.
+
+        Jump here:
+
+        * plainly: start_list
+        * when a ", # or $ is ahead: start_list
+        * from a number followed by , (but don't consume the ,)
+        * from a symbol
+
+        """
         yield SKIP_WHITESPACE
-        yield r'([.,])\s*(\d+)' + RE_LILYPOND_ID_RIGHT_BOUND, bygroup(Separator, Number)
-        yield r'([.,])\s*(' + RE_LILYPOND_ID + r')' + RE_LILYPOND_ID_RIGHT_BOUND, bygroup(Separator, Name.Variable)
-        yield r'([.,])\s*(?=[#$"])', Separator, cls.identifier_scheme_or_string
+        yield r'([.,])\s*(\d+)(?!(?:/|\.\d+))' + RE_LILYPOND_ID_RIGHT_BOUND, bygroup(Separator, Number)
+        yield r'([.,])\s*(' + RE_LILYPOND_ID + r')' + RE_LILYPOND_ID_RIGHT_BOUND, \
+            bygroup(Separator, cls.get_symbol_action(MATCH[2], Name.Variable))
+        yield r'([.,])\s*(?=[#$"])', Separator, cls._continue_list
+        yield from cls.find_comment()
         yield default_target, -1
 
     @lexicon
-    def identifier_scheme_or_string(cls):
-        """Pick a scheme expression or a string.
+    def start_list(cls):
+        """Start a list, this context is never created: all contents go to ``list``."""
+        yield SKIP_WHITESPACE
+        yield from cls.find_string(-1, cls.list)
+        yield from cls.find_scheme(-1, cls.list)
+                 # vvvvvvvvv not a real or fraction
+        yield r'\d+(?!(?:/|\.\d+))' + RE_LILYPOND_ID_RIGHT_BOUND, Number, -1, cls.list
+        yield RE_LILYPOND_SYMBOL, cls.get_symbol_action(TEXT, Name.Variable), -1, cls.list
+        yield default_target, -1
 
-        Immediately pops back, so this context is never actually created.
-
-        """
+    @lexicon
+    def _continue_list(cls):
+        """Continue a list, this context is never created: all contents remain in ``list``."""
         yield from cls.find_string(-1)
         yield from cls.find_scheme(-1)
         yield default_target, -1
 
     @lexicon(consume=True)
-    def unit(cls):
-        """A unit that might occur after a numeric value in a paper block."""
-        yield SKIP_WHITESPACE
-        yield r'\\(mm|in|pt|cm)' + RE_LILYPOND_ID_RIGHT_BOUND, Name.Builtin, -1
-        yield default_target, -1
+    def identifier(cls):
+        """bla.bla.bla syntax."""
+        yield from cls.list
+
+    @lexicon(consume=True)
+    def identifier_ref(cls):
+        r"""\bla.bla.bla syntax."""
+        yield r'(?<=\\)"', String, cls.string   # only after '\'
+        yield from cls.list
+
+    @classmethod
+    def get_symbol_action(self, text, default=Name.Symbol):
+        """Return a proper dynamic action for the name of a symbol."""
+        return findmember(text, (
+                (lilypond_words.grobs, Grob),
+                (lilypond_words.contexts, Context),
+                ), default)
 
     # -------------------- markup --------------------
+    @classmethod
+    def find_markup(cls):
+        r"""Find ``\markup``, ``\markuplines`` and ``\markuplist``."""
+        yield r"\\markup(?:lines|list)?" + RE_LILYPOND_ID_RIGHT_BOUND, Keyword.Markup, cls.markup
+
     @lexicon(consume=True)
     def markup(cls):
         """Markup without environment. Try to guess the n of arguments."""
-        yield r'\{', Bracket.Markup.Start, -1, cls.markuplist
-        yield r"(\\score)\s*(\{)", bygroup(Name.Function.Markup, Bracket.Start), -1, cls.score
-        yield RE_LILYPOND_COMMAND, cls.get_markup_action(), \
-            select(call(cls.get_markup_argument_count, MATCH[1]), -1, 0, 1, 2, 3)
-        yield from cls.find_string(-1)
-        yield from cls.find_scheme(-1)
-        yield from cls.find_comment()
-        yield RE_LILYPOND_MARKUP_TEXT, Text, -1
-
-    @classmethod
-    def get_markup_argument_count(cls, command):
-        """Return the number of arguments the markup command (without \\) expects."""
-        for i in range(5):
-            if command in lilypond_words.markup_commands_nargs[i]:
-                return i
-        return 1    # assume a user command has one argument
+        yield from cls._markup_rules(-1)
 
     @lexicon(consume=True)
     def markuplist(cls):
-        """Markup until } ."""
+        """Markup in environment from ``{`` until ``}``."""
         yield r'\}', Bracket.Markup.End, -1
-        yield r'\{', Bracket.Markup.Start, 1
-        yield r"(\\score)\s*(\{)", bygroup(Name.Function.Markup, Bracket.Start), cls.score
-        yield RE_LILYPOND_COMMAND, cls.get_markup_action()
-        yield from cls.base()
-        yield RE_LILYPOND_MARKUP_TEXT, Text
+        yield from cls._markup_rules()
+
+    @lexicon(consume=True)
+    def markupscore(cls):
+        r"""``\score { }`` or ``\score-lines { }`` in markup."""
+        yield from cls.score
+
+    @classmethod
+    def _markup_rules(cls, *extra_target):
+        r"""Markup rules. Specify ``-1`` if outside an environment."""
+        yield r'\{', Bracket.Markup.Start, *extra_target, cls.markuplist
+        yield r"(\\score(?:-lines)?)\s*(\{)", bygroup(Name.Function.Markup, Bracket.Start), *extra_target, cls.markupscore
+        if extra_target:
+            # count the arguments of markup commands and stack markup contexts
+            yield RE_LILYPOND_COMMAND, cls.get_markup_action(), \
+                select(call(cls.get_markup_argument_count, MATCH[1]), -1, 0, 1, 2, 3)
+        else:
+            # in an environment; no need to count the arguments.
+            yield RE_LILYPOND_COMMAND, cls.get_markup_action()
+        yield from cls.find_string(*extra_target)
+        yield from cls.find_scheme(*extra_target)
+        yield from cls.find_comment()
+        yield RE_LILYPOND_MARKUP_TEXT, Text, *extra_target
+
+    @classmethod
+    def get_markup_argument_count(cls, command):
+        r"""Return the number of arguments of a markup command (without ``\``) expects.
+
+        The default implementation returns 1 for unknown commands. You could
+        re-implement this method if you have some special commands to add.
+
+        """
+        try:
+            return lilypond_words.markup_commands[command]
+        except KeyError:
+            return 1    # assume a user command has one argument
 
     @classmethod
     def get_markup_action(cls):
@@ -504,9 +585,9 @@ class LilyPond(Language):
 
     # -------------- Scheme ---------------------
     @classmethod
-    def find_scheme(cls, extra_target=0):
+    def find_scheme(cls, *extra_target):
         """Find scheme."""
-        yield r'[#$]', Delimiter.ModeChange.SchemeStart, extra_target, cls.get_scheme_target()
+        yield r'[#$]', Delimiter.ModeChange.SchemeStart, *extra_target, cls.get_scheme_target()
 
     @classmethod
     def get_scheme_target(cls):
@@ -522,9 +603,9 @@ class LilyPond(Language):
 
     # -------------- String ---------------------
     @classmethod
-    def find_string(cls, extra_target=0):
+    def find_string(cls, *extra_target):
         """Find a string."""
-        yield '"', String, extra_target, cls.string
+        yield '"', String, *extra_target, cls.string
 
     @lexicon(consume=True)
     def string(cls):
@@ -535,10 +616,10 @@ class LilyPond(Language):
 
     # -------------- Comment ---------------------
     @classmethod
-    def find_comment(cls):
+    def find_comment(cls, *extra_target):
         """Find single-line or block comments."""
-        yield r'%\{', Comment, cls.multiline_comment
-        yield r'%', Comment, cls.singleline_comment
+        yield r'%\{', Comment, *extra_target, cls.multiline_comment
+        yield r'%', Comment, *extra_target, cls.singleline_comment
 
     @lexicon(consume=True)
     def multiline_comment(cls):
