@@ -28,7 +28,9 @@ Some lists could maybe be generated dynamically.
 keywords = (
     'accepts',
     'alias',
+    'change',
     'consists',
+    'context',
     'defaultchild',
     'denies',
     #'description',
@@ -38,6 +40,7 @@ keywords = (
     #'invalid',
     'language',
     'name',
+    'new',
     #'objectid',
     'omit', # since 2.18
     'once',
@@ -52,7 +55,21 @@ keywords = (
     #'type',
     'undo', # since 2.18 (not mentioned in the command index)
     'version',
-    'score',
+
+    # input modes
+    'addlyrics',
+    'lyricmode',
+    'lyrics',
+    'lyricsto',
+    'chordmode',
+    'chords',
+    'drummode',
+    'drums',
+    'figuremode',
+    'figures',
+    'notemode',
+
+    # environments:
     'book',
     'bookpart',
     'header',
@@ -60,204 +77,9 @@ keywords = (
     'midi',
     'layout',
     'with',
-    'context',
-)
-
-
-markup_commands_nargs = (
-# no arguments
-(
-    'doubleflat',
-    'doublesharp',
-    'eyeglasses',
-    'flat',
-    'natural',
-    'null',
-    'semiflat',
-    'semisharp',
-    'sesquiflat',
-    'sesquisharp',
-    'sharp',
-    'strut',
-    'table-of-contents'
-),
-# one argument
-(
-    'backslashed-digit',
-    'bold',
-    'box',
-    'bracket',
-    'caps',
-    'center-align',
-    'center-column',
-    'char',
-    'circle',
-    'column',
-    'concat',
-    'dir-column',
-    'draw-dashed-line', # since 2.18
-    'draw-dotted-line', # since 2.18
-    'draw-line',
-    'dynamic',
-    'fill-line',
-    'finger',
-    'fontCaps',
-    'fret-diagram',
-    'fret-diagram-terse',
-    'fret-diagram-verbose',
-    'fromproperty',
-    'harp-pedal',
-    'hbracket',
-    'hspace',
-    'huge',
-    'italic',
-    'justify',
-    'justify-field',
-    'justify-string',
-    'large',
-    'larger',
-    'left-align',
-    'left-brace',
-    'left-column',
-    'line',
-    'lookup',
-    'markalphabet',
-    'markletter',
-    'medium',
-    'musicglyph',
-    'normalsize',
-    'normal-size-sub',
-    'normal-size-super',
-    'normal-text',
-    'number',
-    'oval', # since 2.18
-    'postscript',
-    'right-align',
-    'right-brace',
-    'right-column',
-    'roman',
-    'rounded-box',
-    'sans',
     'score',
-    'simple',
-    'slashed-digit',
-    'small',
-    'smallCaps',
-    'smaller',
-    'stencil',
-    'sub',
-    'super',
-    'teeny',
-    'text',
-    'tied-lyric',
-    'tiny',
-    'transparent',
-    'triangle',
-    'typewriter',
-    'underline',
-    'upright',
-    'vcenter',
-    'vspace',
-    'verbatim-file',
-    'whiteout',
-    'wordwrap',
-    'wordwrap-field',
-    'wordwrap-string',
-),
-# two arguments
-(
-    'abs-fontsize',
-    'auto-footnote', # since 2.16
-    'combine',
-    'customTabClef',
-    'fontsize',
-    'footnote',
-    'fraction',
-    'halign',
-    'hcenter-in',
-    'lower',
-    'magnify',
-    'note',
-    'on-the-fly',
-    'override',
-    'pad-around',
-    'pad-markup',
-    'pad-x',
-    'page-link',
-    'path',     # added in LP 2.13.31
-    'raise',
-    'rotate',
-    'scale',
-    'translate',
-    'translate-scaled',
-    'with-color',
-    'with-link',
-    'with-url',
-    'woodwind-diagram',
-),
-# three arguments
-(
-    'arrow-head',
-    'beam',
-    'draw-circle',
-    'epsfile',
-    'filled-box',
-    'general-align',
-    'note-by-number',
-    'pad-to-box',
-    'page-ref',
-    'with-dimensions',
-),
-# four arguments
-(
-    'pattern',
-    'put-adjacent',
-),
-# five arguments,
-(
-    'fill-with-pattern',
-),
 )
 
-markup_commands = sum(markup_commands_nargs, ())
-
-dynamics = (
-    "n",    # niente, since 2.22
-    "ppppp",
-    "pppp",
-    "ppp",
-    "pp",
-    "p",
-    "mp",
-    "mf",
-    "f",
-    "ff",
-    "fff",
-    "ffff",
-    "fffff",
-
-    "fp",
-    "sf",
-    "sfp",
-    "sff",
-    "sfz",
-    "fz",   # undocumented
-    "sp",
-    "spp",
-    "rfz",
-
-    # dynamic spanners:
-    "cr",
-    "decr",
-    "enddecr",
-    "endcr",
-    "cresc",
-    "endcresc",
-    "dim",
-    "enddim",
-    "decresc",
-    "enddecresc",
-)
 
 music_commands = (
     'absolute', # since 2.18
@@ -265,7 +87,6 @@ music_commands = (
     'accidentalStyle', # since 2.16
     'addChordShape', # since 2.16
     'addInstrumentDefinition',
-    'addlyrics',
     'addQuote',
     'afterGrace',
     #'afterGraceFraction', # this is a parser variable
@@ -280,7 +101,6 @@ music_commands = (
     'applyMusic',
     'applyOutput',
     'appoggiatura',
-    'arpeggio',
     'arpeggioArrowDown',
     'arpeggioArrowUp',
     'arpeggioBracket',
@@ -312,23 +132,18 @@ music_commands = (
     'bracketOpenSymbol',
     'break',
     'breathe',
-    'breve',
     'cadenzaOff',
     'cadenzaOn',
     'caesura',
     'cavum',
-    'change',
-    'chordmode',
     #'chordNameSeparator',
     #'chordPrefixSpacer',
     #'chordRootNamer',
     'chordRepeats', # since 2.16
-    'chords',
     'clef',
     'cm',
     'compoundMeter', # since 2.16
     'compressFullBarRests',
-    'context',
     'crescHairpin',
     'crescTextCresc',
     'crossStaff', # since 2.16
@@ -349,7 +164,6 @@ music_commands = (
     'defaultTimeSignature',
     'defineBarLine', # since 2.18
     'deminutum',
-    'denies',
     'descendens',
     'dimHairpin',
     'dimTextDecr',
@@ -363,9 +177,7 @@ music_commands = (
     'dotsDown',
     'dotsNeutral',
     'dotsUp',
-    'drummode',
     'drumPitchTable',
-    'drums',
     'dynamicDown',
     'dynamicNeutral',
     'dynamicUp',
@@ -383,17 +195,15 @@ music_commands = (
     'expandFullBarRests',
     'featherDurations',
     'fermataMarkup',
-    'figuremode',
-    'figures',
     'finalis',
     'fingeringOrientations',
+    'fixed',    # since 2.20
     'flexa',
     'frenchChords',
     'fullJazzExceptions',
     'funkHeads',
     'funkHeadsMinor',
     'germanChords',
-    'glissando',
     'grace',
     'graceSettings',
     'harmonic',
@@ -420,14 +230,9 @@ music_commands = (
     'key',
     'killCues',
     'label',
-    'laissezVibrer',
     'large',
     'ligature',
     'linea',
-    'longa',
-    'lyricmode',
-    'lyrics',
-    'lyricsto',
     'maininput',
     'majorSevenSymbol',
     'makeClusters',
@@ -437,9 +242,6 @@ music_commands = (
     'markup',
     'markuplines', # deprecated, till 2.14
     'markuplist', # from 2.16
-    'maxima',
-    'melisma',
-    'melismaEnd',
     'mergeDifferentlyDottedOff',
     'mergeDifferentlyDottedOn',
     'mergeDifferentlyHeadedOff',
@@ -447,21 +249,18 @@ music_commands = (
     'mm',
     'musicMap',
     'neumeDemoLayout',
-    'new',
     'newSpacingSection',
     'noBeam',
     'noBreak',
     'noPageBreak',
     'noPageTurn',
     'normalsize',
-    'notemode',
     'numericTimeSignature',
     'octaveCheck',
     'oldaddlyrics',
     'oneVoice',
     'oriscus',
     'ottava',
-    'override',
     'overrideProperty',
     'overrideTimeSignatureSettings',  # since 2.16
     'pageBreak',
@@ -499,18 +298,15 @@ music_commands = (
     'RemoveEmptyStaffContext',
     'removeWithTag',
     'repeat',
-    'repeatTie',
     'resetRelativeOctave',
     'responsum',
     'rest',
-    'revert',
     'rightHandFinger',
     'sacredHarpHeads',
     'sacredHarpHeadsMinor',
     'scaleDurations',
     'scoreTweak',
     'semiGermanChords',
-    'set',
     'shape', # since 2.16
     'shiftDurations',
     'shiftOff',
@@ -553,6 +349,7 @@ music_commands = (
     'sustainOn',
     'tabFullNotation',
     'tag',
+    'tagGroup',     # since 2.20
     'teeny',
     'tempo',
     'tempoWholesPerMinute',
@@ -569,7 +366,7 @@ music_commands = (
     'tieUp',
     'tildeSymbol',
     'time',
-    'times',
+    'times',    # not mentioned since 2.18 (superseded by \tuplet)
     'timing',
     'tiny',
     'tocItem', # since ?
@@ -586,7 +383,6 @@ music_commands = (
     'unfoldRepeats',
     'unHideNotes',
     'unit',
-    'unset',
     'versus',
     'virga',
     'virgula',
@@ -606,6 +402,234 @@ music_commands = (
 )
 
 
+markup_commands_nargs = (
+# no arguments
+(
+    'doubleflat',
+    'doublesharp',
+    'draw-hline',       # since 2.14
+    'eyeglasses',
+    'flat',
+    'natural',
+    'null',
+    'semiflat',
+    'semisharp',
+    'sesquiflat',
+    'sesquisharp',
+    'sharp',
+    'strut',
+    'table-of-contents'
+),
+# one argument
+(
+    'backslashed-digit',
+    'bold',
+    'box',
+    'bracket',
+    'caps',
+    'center-align',
+    'center-column',
+    'char',
+    'circle',
+    'column',
+    'column-lines',     # since 2.14
+    'concat',
+    'compound-meter',   # since 2.22
+    'dir-column',
+    'discant',          # since 2.20
+    'draw-dashed-line', # since 2.18
+    'draw-dotted-line', # since 2.18
+    'draw-line',
+    'dynamic',
+    'ellipse',          # since 2.20
+    'fill-line',
+    'finger',
+    'first-visible',    # since 2.20
+    'fontCaps',
+    'freeBass',         # since 2.20 #(use-modules (scm accreg))
+    'fret-diagram',
+    'fret-diagram-terse',
+    'fret-diagram-verbose',
+    'fromproperty',
+    'harp-pedal',
+    'hbracket',
+    'hspace',
+    'huge',
+    'italic',
+    'justify',
+    'justify-field',
+    'justify-line',     # since 2.20
+    'justify-string',
+    'large',
+    'larger',
+    'left-align',
+    'left-brace',
+    'left-column',
+    'line',
+    'lookup',
+    'markalphabet',
+    'markletter',
+    'medium',
+    'musicglyph',
+    'normalsize',
+    'normal-size-sub',
+    'normal-size-super',
+    'normal-text',
+    'number',
+    'oval',         # since 2.18
+    'overlay',      # since 2.20
+    'overtie',      # since 2.20
+    'parenthesize', # since 2.14
+    'postscript',
+    'property-recursive',   # since 2.16
+    'rest',         # since 2.20
+    'right-align',
+    'right-brace',
+    'right-column',
+    'roman',
+    'rounded-box',
+    'sans',
+    'score',
+    'score-lines',  # since 2.20
+    'simple',
+    'slashed-digit',
+    'small',
+    'smallCaps',
+    'smaller',
+    'stdBase',      # since 2.20 #(use-modules (scm accreg))
+    'stdBaseIV',    # since 2.20 #(use-modules (scm accreg))
+    'stdBaseV',     # since 2.20 #(use-modules (scm accreg))
+    'stdBaseVI',    # since 2.20 #(use-modules (scm accreg))
+    'stencil',
+    'sub',
+    'super',
+    'teeny',
+    'text',
+    'tie',          # since 2.20
+    'tied-lyric',
+    'tiny',
+    'transparent',
+    'triangle',
+    'typewriter',
+    'underline',
+    'undertie',     # since 2.20
+    'upright',
+    'vcenter',
+    'vspace',
+    'verbatim-file',
+    'whiteout',
+    'wordwrap',
+    'wordwrap-field',
+    'wordwrap-string',
+),
+# two arguments
+(
+    'abs-fontsize',
+    'auto-footnote', # since 2.16
+    'combine',
+    'customTabClef',
+    'fontsize',
+    'footnote',
+    'fraction',
+    'halign',
+    'hcenter-in',
+    'lower',
+    'magnify',
+    'map-markup-commands',  # since 2.20
+    'note',
+    'on-the-fly',
+    'override',
+    'override-lines',       # since 2.12
+    'pad-around',
+    'pad-markup',
+    'pad-x',
+    'page-link',
+    'path',         # since 2.14
+    'raise',
+    'replace',      # since 2.20
+    'rest-by-number',   # since 2.20
+    'rotate',
+    'scale',
+    'table',        # since 2.20
+    'translate',
+    'translate-scaled',
+    'with-color',
+    'with-dimensions-from', # since 2.20
+    'with-link',
+    'with-outline',     # since 2.20
+    'with-url',
+    'woodwind-diagram',
+),
+# three arguments
+(
+    'arrow-head',
+    'beam',
+    'draw-circle',
+    'draw-squiggle-line',   # since 2.20
+    'epsfile',
+    'filled-box',
+    'general-align',
+    'note-by-number',
+    'pad-to-box',
+    'page-ref',
+    'with-dimensions',
+),
+# four arguments
+(
+    'pattern',
+    'put-adjacent',
+),
+# five arguments,
+(
+    'fill-with-pattern',
+),
+)
+
+markup_commands = {}
+for n, cmds in enumerate(markup_commands_nargs):
+    markup_commands.update(dict.fromkeys(cmds, n))
+del n, cmds
+
+
+dynamics = (
+    "n",    # niente, since 2.22
+    "ppppp",
+    "pppp",
+    "ppp",
+    "pp",
+    "p",
+    "mp",
+    "mf",
+    "f",
+    "ff",
+    "fff",
+    "ffff",
+    "fffff",
+
+    "fp",
+    "sf",
+    "sfp",
+    "sff",
+    "sfz",
+    "fz",   # undocumented
+    "sp",
+    "spp",
+    "rfz",
+
+    # dynamic spanners:
+    "cr",
+    "decr",
+    "enddecr",
+    "endcr",
+    "cresc",
+    "endcresc",
+    "dim",
+    "enddim",
+    "decresc",
+    "enddecresc",
+)
+
+
 articulations = (
     'accent',
     'espressivo',
@@ -686,6 +710,12 @@ spanner_events = (
     'stopTextSpan',
     'startTrillSpan',
     'stopTrillSpan',
+    'repeatTie',
+    'laissezVibrer',
+    'melisma',
+    'melismaEnd',
+    'arpeggio',
+    'glissando',
 )
 
 
